#!/usr/bin/env python

"""
HTML serialiser.

Copyright (C) 2017, 2018 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from moinformat.serialisers.common import escape_attr, escape_text, Serialiser

class HTMLSerialiser(Serialiser):

    "Serialisation of the page."

    format = "html"

    def _region_tag(self, type):

        # NOTE: Need to support types in general.

        type = type and type.split()[0]

        if type == "inline":
            return "tt"
        elif type in (None, "python"):
            return "pre"
        else:
            return "span"

    def start_region(self, level, indent, type, extra):

        # Generate attributes, joining them when preparing the tag.

        l = []
        out = l.append

        if level:
            out("level-%d" % level)

        if indent:
            out("indent-%d" % indent)

        # NOTE: Encode type details for CSS.

        out("type-%s" % escape_attr(type or "opaque"))

        tag = self._region_tag(type)
        self.out("<%s class='%s'>" % (tag, " ".join(l)))

    def end_region(self, level, indent, type, extra):
        tag = self._region_tag(type)
        self.out("</%s>" % tag)

    def start_block(self):
        self.out("<p>")

    def end_block(self):
        self.out("</p>")

    def start_defitem(self, pad, extra):
        self.out("<dd>")

    def end_defitem(self, pad, extra):
        self.out("</dd>")

    def start_defterm(self, pad, extra):
        self.out("<dt>")

    def end_defterm(self, pad, extra):
        self.out("</dt>")

    def start_emphasis(self):
        self.out("<em>")

    def end_emphasis(self):
        self.out("</em>")

    def start_heading(self, level, extra, pad, identifier):
        self.out("<h%d id='%s'>" % (level, escape_attr(self.linker.make_id(identifier))))

    def end_heading(self, level, pad, extra):
        self.out("</h%d>" % level)

    def start_larger(self):
        self.out("<big>")

    def end_larger(self):
        self.out("</big>")

    def start_link(self, target, nodes):
        label = None
        if self.linker:
            target, label = self.linker.translate(target)
        self.out('<a href="%s">' % escape_attr(target))
        if not nodes:
            self.out(escape_text(label or target))

    def end_link(self):
        self.out("</a>")

    def start_linktext(self):
        pass

    def end_linktext(self):
        pass

    list_tags = {
        "i" : "lower-roman",
        "I" : "upper-roman",
        "a" : "lower-latin",
        "A" : "upper-latin",
        }

    def _get_list_tag(self, marker):
        if marker:
            if marker[0].isdigit():
                return "ol", "decimal"
            style_type = self.list_tags.get(marker[0])
            if style_type:
                return "ol", style_type

        return "ul", None

    def start_list(self, indent, marker, num):
        tag, style_type = self._get_list_tag(marker)
        style = style_type and ' style="list-style-type: %s"' % escape_attr(style_type) or ""
        start = style_type and num is not None and ' start="%s"' % escape_attr(num) or ""
        self.out("<%s%s%s>" % (tag, style, start))

    def end_list(self, indent, marker, num):
        tag, style = self._get_list_tag(marker)
        self.out("</%s>" % tag)

    def start_listitem(self, indent, marker, space, num):
        self.out("<li>")

    def end_listitem(self, indent, marker, space, num):
        self.out("</li>")

    def start_macro(self, name, args, nodes):
        self.out("<span class='macro'>")

        # Fallback case for when macros are not replaced.

        if not nodes:
            self.out(escape_text("<<"))
            self.out("<span class='name'>%s</span>" % escape_text(name))
            if args:
                self.out("(")
            first = True
            for arg in args:
                if not first:
                    self.out(",")
                self.out("<span class='arg'>%s</span>" % escape_text(arg))
                first = False
            if args:
                self.out(")")
            self.out(escape_text(">>"))

    def end_macro(self):
        self.out("</span>")

    def start_monospace(self):
        self.out("<tt>")

    def end_monospace(self):
        self.out("</tt>")

    def start_smaller(self):
        self.out("<small>")

    def end_smaller(self):
        self.out("</small>")

    def start_strikethrough(self):
        self.out("<del>")

    def end_strikethrough(self):
        self.out("</del>")

    def start_strong(self):
        self.out("<strong>")

    def end_strong(self):
        self.out("</strong>")

    def start_subscript(self):
        self.out("<sub>")

    def end_subscript(self):
        self.out("</sub>")

    def start_superscript(self):
        self.out("<sup>")

    def end_superscript(self):
        self.out("</sup>")

    def start_table(self):
        self.out("<table>")

    def end_table(self):
        self.out("</table>")

    def start_table_attrs(self):
        pass

    def end_table_attrs(self):
        pass

    def start_table_cell(self, attrs):
        self.out("<td")

        # Handle the attributes separately from their container.

        if attrs and not attrs.empty():
            for attr in attrs.nodes:
                attr.to_string(self)

        self.out(">")

    def end_table_cell(self):
        self.out("</td>")

    def start_table_row(self):
        self.out("<tr>")

    def end_table_row(self, trailing):
        self.out("</tr>")

    def start_underline(self):
        self.out("<span style='text-decoration: underline'>")

    def end_underline(self):
        self.out("</span>")

    def anchor(self, target):
        self.out("<a name='%s' />" % escape_attr(self.linker.make_id(target)))

    def break_(self):
        pass

    def linebreak(self):
        self.out("<br />")

    def rule(self, length):
        self.out("<hr style='height: %dpt' />" % min(length, 10))

    def table_attrs(self, nodes):

        # Skip the attributes in their original form.

        pass

    def table_attr(self, name, value, concise, quote):
        self.out(" %s%s" % (escape_text(name), value is not None and
            "='%s'" % escape_attr(value) or ""))

    def text(self, s):
        self.out(escape_text(s))

serialiser = HTMLSerialiser

# vim: tabstop=4 expandtab shiftwidth=4
